[DEFAULT]
debug = true
{% if publisher %}
namespace = kolla
tag = {{ zuul.tag if zuul.pipeline == "release" else zuul.branch | basename }}
{% endif %}
push = false
logs_dir = /tmp/logs/build
template_override = /etc/kolla/template_overrides.j2
