{% extends parent_template %}

{% block opendaylight_install %}
  {% if base_distro in ['centos', 'oraclelinux', 'rhel'] %}
    {% set opendaylight_packages = ['java-1.8.0-openjdk'] %}
  {% elif base_distro in ['ubuntu', 'debian'] %}
    {% set opendaylight_packages = ['openjdk-8-jre'] %}
  {% endif %}

{{ macros.install_packages(opendaylight_packages | customizable("packages")) }}

ADD opendaylight-archive /opendaylight
RUN ln -s /opendaylight/* /opt/opendaylight
{% endblock %}
